// Name: 4 Point Sphere
// File: aw4PointSphere.ls
// Author: Artur Wawrzonkiewicz
// Version: 1.0
// Latest update: 02/01/11

// Based on:
// http://local.wasp.uwa.edu.au/~pbourke/geometry/spherefrom4/
// http://www.euclideanspace.com/maths/algebra/matrix/functions/determinant/fourD/index.htm

@warnings
@name 4 Point Sphere
@version 2.6
@script modeler

//Initialize determinant matrix
var a[1][1] = 0.0;

//Determinants
var m11,m2,m13,m14,m15;

main
{
	var c = <0,0,0>;
	var r = 0.0;
	
	var sides = 24;
	var segments = 12;
	var level = 4;
	
	selmode(DIRECT);

	pntCnt = pointcount();

	if(pntCnt <> 4) 
	{        
		error("4 points are required.");
	}

	editbegin();

	// Find determinant M11 
	for (i = 1 ; i <= 4 ; i++)
	{
		a[i][1] = points[i].x;
		a[i][2] = points[i].y;
		a[i][3] = points[i].z;
		a[i][4] = 1;
	}
	m11 = Determinant();

   	// Find determinant M12 
	for (i = 1 ; i <= 4 ; i++)
	{
	      a[i][1] = points[i].x * points[i].x + points[i].y * points[i].y + points[i].z * points[i].z;
	      a[i][2] = points[i].y;
	      a[i][3] = points[i].z;
	      a[i][4] = 1;
   	}
   	m12 = Determinant();

      // Find determinant M13     
	for (i = 1 ; i <= 4 ; i++)
	{
	      a[i][1] = points[i].x;
	      a[i][2] = points[i].x * points[i].x + points[i].y * points[i].y + points[i].z * points[i].z;
	      a[i][3] = points[i].z;
	      a[i][4] = 1;
   	}
   	m13 = Determinant();

   	// Find determinant M14     
   	for (i = 1 ; i <= 4 ; i++)
   	{
	      a[i][1] = points[i].x;
	      a[i][2] = points[i].y;
	      a[i][3] = points[i].x * points[i].x + points[i].y * points[i].y + points[i].z * points[i].z;
	      a[i][4] = 1;
   	}
   	m14 = Determinant();

   	// Find determinant M15
   	for (i = 1 ; i <= 4 ; i++)
   	{
	      a[i][1] = points[i].x * points[i].x + points[i].y * points[i].y + points[i].z * points[i].z;
	      a[i][2] = points[i].x;
	      a[i][3] = points[i].y;
	      a[i][4] = points[i].z;
	}
   	m15 = Determinant();
	
  	if (m11 == 0) 
   	{
	      error("The points don't define a sphere.");
   	}

   	c.x = 0.5 * m12 / m11;
   	c.y = 0.5 * m13 / m11;
   	c.z = 0.5 * m14 / m11;
   	
   	r = sqrt(c.x*c.x + c.y*c.y + c.z*c.z - m15/m11);
	
	editend();
	
	reqbegin("4 Point Sphere");

	c1 = ctlchoice("Type",0,@"Globe","Tesselation"@,false);
	c2 = ctlinteger("Sides", sides);
	c3 = ctlinteger("Segments", segments);
	c4 = ctlinteger("Tesselation level", level);
	if(!reqpost()) return;

	type = getvalue(c1);
	sides = getvalue(c2);
	segments = getvalue(c3);
	level = getvalue(c4);
	
	reqend();
	
	if(level < 1) level = 1;
	if(sides < 3) sides = 3;
	if(segments < 2) segments = 2;
	
	if(type == 1)
		makeball(r,sides, segments,c);
	else
		maketesball(r,level,c);
}

Determinant
{
      var det = 
      a[1][4] * a[2][3] * a[3][2] * a[4][1]-a[1][3] * a[2][4] * a[3][2] * a[4][1]-a[1][4] * a[2][2] * a[3][3] * a[4][1]+a[1][2] * a[2][4] * a[3][3] * a[4][1]+
      a[1][3] * a[2][2] * a[3][4] * a[4][1]-a[1][2] * a[2][3] * a[3][4] * a[4][1]-a[1][4] * a[2][3] * a[3][1] * a[4][2]+a[1][3] * a[2][4] * a[3][1] * a[4][2]+
      a[1][4] * a[2][1] * a[3][3] * a[4][2]-a[1][1] * a[2][4] * a[3][3] * a[4][2]-a[1][3] * a[2][1] * a[3][4] * a[4][2]+a[1][1] * a[2][3] * a[3][4] * a[4][2]+
      a[1][4] * a[2][2] * a[3][1] * a[4][3]-a[1][2] * a[2][4] * a[3][1] * a[4][3]-a[1][4] * a[2][1] * a[3][2] * a[4][3]+a[1][1] * a[2][4] * a[3][2] * a[4][3]+
      a[1][2] * a[2][1] * a[3][4] * a[4][3]-a[1][1] * a[2][2] * a[3][4] * a[4][3]-a[1][3] * a[2][2] * a[3][1] * a[4][4]+a[1][2] * a[2][3] * a[3][1] * a[4][4]+
      a[1][3] * a[2][1] * a[3][2] * a[4][4]-a[1][1] * a[2][3] * a[3][2] * a[4][4]-a[1][2] * a[2][1] * a[3][3] * a[4][4]+a[1][1] * a[2][2] * a[3][3] * a[4][4];
      
      return det;
}
